import NDK, { NDKEvent, type NDKEventId, NDKKind, type NDKUser } from '@nostr-dev-kit/ndk';
import { isValidNip05 } from '$lib/utils.svelte';
import { TokenInfoWithMailSubscriptionDuration } from '@arx/utils';

export class Letter {
	id: NDKEventId;
	subject: string;
	from: NDKUser;
	content: string;
	date: Date;
	recipients: Set<NDKUser>;
	stamp?: TokenInfoWithMailSubscriptionDuration;
	emailAddress?: string;

	private constructor() {
		this.id = '';
		this.subject = '';
		this.from = {} as NDKUser;
		this.content = '';
		this.date = new Date();
		this.recipients = new Set<NDKUser>();
	}

	static async fromDecryptedMessage(
		decryptedMessage: NDKEvent,
		encryptedMessage: NDKEvent,
		ndk: NDK
	): Promise<Letter> {
		if (decryptedMessage.kind !== NDKKind.Article) throw new Error('Not a letter');

		const letter = new Letter();
		letter.id = encryptedMessage.id;
		letter.subject = decryptedMessage.tags.find((t) => t[0] === 'subject')?.[1] ?? 'No subject';
		letter.from = decryptedMessage.author;
		letter.content = decryptedMessage.content;
		letter.date = new Date(decryptedMessage.created_at! * 1000);
		letter.emailAddress = decryptedMessage.tags.find((t) => t[0] === 'email:from')?.[1];
		let stampToken = decryptedMessage.tags.find((t) => t[0] === 'stamp')?.[1];
		if (stampToken) {
			try {
				stampToken = await ndk.signer!.nip44Decrypt(letter.from, stampToken);
				letter.stamp = new TokenInfoWithMailSubscriptionDuration(stampToken);
			} catch (error) {
				console.error('Error decrypting stamp token:', error);
			}
		}

		for (const tag of decryptedMessage.tags) {
			if (tag[0] === 'p') {
				try {
					if (isValidNip05(tag[1])) {
						const parsed = await ndk.getUserFromNip05(tag[1]);
						if (parsed) letter.recipients.add(parsed);
					} else if (tag[1].startsWith('npub')) {
						letter.recipients.add(ndk.getUser({ npub: tag[1] }));
					} else {
						letter.recipients.add(ndk.getUser({ pubkey: tag[1] }));
					}
				} catch (error) {
					console.error('Error processing recipient:', error);
				}
			}
		}

		return letter;
	}
}
